
# q-expansion of newform 171.4.f.h, downloaded from the LMFDB on 14 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 171
weight = 4
poly_data = [8856576, 0, 2092128, 0, 345409, 0, 29198, 0, 1797, 0, 50, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[777219275616, 0, 128318407273, 0, 9091814878, 0, 667580109, 0, 18574850, 0, 421097, 0], 511964824416], [[0, -265254451200, 0, -128318407273, 0, -9091814878, 0, -667580109, 0, -18574850, 0, -421097], 511964824416], [[5014036725, 0, -104606400, 0, -17270450, 0, -3229209, 0, -89850, 0, -2500, 0], 516093573], [[0, 7594504590, 0, -104606400, 0, -17270450, 0, -3229209, 0, -89850, 0, -2500], 516093573], [[-9483250101600, 0, -1565678550425, 0, -137428566526, 0, -8145486525, 0, -226641250, 0, -4678649, 0], 511964824416], [[0, 86318337156768, 0, 14251102475079, 0, 1557563627618, 0, 74141759907, 0, 2062931550, 0, 28392391], 3071788946496], [[0, 6684673239840, 0, 1103635293395, 0, 91443820458, 0, 5741693535, 0, 159757750, 0, 3392067], 255982412208], [[86318337156768, 0, 14251102475079, 0, 1557563627618, 0, 74141759907, 0, 2062931550, 0, 28392391, 0], 1535894473248], [[0, 28135631979, 0, -2713490016, 0, -447995473, 0, -73443810, 0, -2330709, 0, -64850], 3096561438], [[28135631979, 0, -2713490016, 0, -447995473, 0, -73443810, 0, -2330709, 0, -64850, 0], 1548280719]]

hecke_ring_character_values = [[20, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [154, [-1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0], [-5, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 1], [0, 0, 0, -9, 0, 0, 0, -1, 0, 0, 1, 0], [29, 0, -29, 0, 4, 0, 4, 0, 0, 2, 0, -2], [0, 6, 0, 0, 0, -2, 0, 0, 0, 0, 4, 0], [25, 0, 4, 0, -3, 0, 2, 0, 0, 8, 0, -7], [0, 5, 0, -5, 0, 0, 0, 1, 2, 0, 0, 0], [0, 10, 0, -10, 0, 0, 0, -10, 0, 0, 0, 0], [17, 0, 0, 0, 15, 0, 0, 0, 0, 0, 0, -17], [47, 0, 0, 0, -18, 0, 0, 0, 0, 0, 0, 34], [0, 16, 0, 0, 0, -6, 0, 0, 0, 0, -2, 0], [0, 0, 121, 0, 0, 0, 15, 0, 0, -27, 0, 0], [0, 24, 0, -24, 0, 0, 0, -6, -2, 0, 0, 0], [0, 9, 0, -9, 0, 0, 0, -5, -14, 0, 0, 0], [0, 27, 0, 0, 0, 4, 0, 0, 0, 0, 29, 0], [567, 0, -567, 0, -4, 0, -4, 0, 0, -26, 0, 26], [475, 0, -475, 0, -39, 0, -39, 0, 0, 21, 0, -21], [0, 150, 0, 0, 0, -2, 0, 0, 0, 0, 24, 0], [0, 0, 315, 0, 0, 0, 40, 0, 0, -60, 0, 0], [0, 0, -481, 0, 0, 0, 11, 0, 0, 5, 0, 0], [0, 0, 0, -22, 0, -2, 0, -44, 2, 0, 44, 0], [0, -145, 0, 145, 0, 0, 0, -1, 8, 0, 0, 0], [0, 0, -180, 0, 0, 0, 22, 0, 0, -28, 0, 0], [0, -146, 0, 146, 0, 0, 0, -24, -6, 0, 0, 0], [1235, 0, 0, 0, -9, 0, 0, 0, 0, 0, 0, 29], [0, 0, 0, -118, 0, 4, 0, 82, -4, 0, -82, 0], [0, 0, -656, 0, 0, 0, -90, 0, 0, -44, 0, 0], [0, 0, 0, 21, 0, 6, 0, 49, -6, 0, -49, 0], [588, 0, -588, 0, -28, 0, -28, 0, 0, 108, 0, -108], [0, -166, 0, 0, 0, 38, 0, 0, 0, 0, 28, 0], [0, -282, 0, 282, 0, 0, 0, 54, -20, 0, 0, 0], [123, 0, -123, 0, 47, 0, 47, 0, 0, -83, 0, 83], [0, -165, 0, 0, 0, 34, 0, 0, 0, 0, -63, 0], [-444, 0, 0, 0, 16, 0, 0, 0, 0, 0, 0, -256], [0, 0, 945, 0, 0, 0, 166, 0, 0, 286, 0, 0], [-1459, 0, 0, 0, -237, 0, 0, 0, 0, 0, 0, 85], [0, -303, 0, 303, 0, 0, 0, 61, 10, 0, 0, 0], [0, -70, 0, 0, 0, -40, 0, 0, 0, 0, 110, 0], [0, 0, 0, -77, 0, 20, 0, 107, -20, 0, -107, 0], [-596, 0, 596, 0, 114, 0, 114, 0, 0, 104, 0, -104], [0, 0, 0, 651, 0, 42, 0, -87, -42, 0, 87, 0], [0, 0, 159, 0, 0, 0, -74, 0, 0, -72, 0, 0], [0, 0, 0, 149, 0, -20, 0, 75, 20, 0, -75, 0], [399, 0, -399, 0, -91, 0, -91, 0, 0, 35, 0, -35], [0, 0, 607, 0, 0, 0, -255, 0, 0, 203, 0, 0], [0, 0, 135, 0, 0, 0, 211, 0, 0, -119, 0, 0], [0, 0, 0, 285, 0, -6, 0, 183, 6, 0, -183, 0], [-2453, 0, 0, 0, -280, 0, 0, 0, 0, 0, 0, 10], [0, 898, 0, 0, 0, 66, 0, 0, 0, 0, 28, 0], [0, 0, 0, -735, 0, -84, 0, 57, 84, 0, -57, 0], [-733, 0, 733, 0, 232, 0, 232, 0, 0, -196, 0, 196], [0, -660, 0, 660, 0, 0, 0, -332, 16, 0, 0, 0], [0, 1015, 0, -1015, 0, 0, 0, 53, 66, 0, 0, 0], [0, -54, 0, 0, 0, -50, 0, 0, 0, 0, 28, 0], [0, 123, 0, 0, 0, -74, 0, 0, 0, 0, 41, 0], [0, 0, -3664, 0, 0, 0, -28, 0, 0, -168, 0, 0], [1872, 0, 0, 0, 442, 0, 0, 0, 0, 0, 0, -96], [0, 655, 0, -655, 0, 0, 0, 71, 12, 0, 0, 0], [0, 0, -4080, 0, 0, 0, -164, 0, 0, -244, 0, 0], [0, 0, 0, -256, 0, -56, 0, 176, 56, 0, -176, 0], [0, 0, -898, 0, 0, 0, 464, 0, 0, 210, 0, 0], [0, 0, 0, -451, 0, 94, 0, -221, -94, 0, 221, 0], [-2780, 0, 2780, 0, -558, 0, -558, 0, 0, 152, 0, -152], [0, -81, 0, 81, 0, 0, 0, 401, 22, 0, 0, 0], [-257, 0, 0, 0, 683, 0, 0, 0, 0, 0, 0, -85], [0, 0, 3385, 0, 0, 0, 342, 0, 0, 472, 0, 0], [0, -867, 0, 0, 0, 52, 0, 0, 0, 0, 415, 0], [-5931, 0, 0, 0, -8, 0, 0, 0, 0, 0, 0, 674], [0, 0, 0, -1085, 0, 182, 0, 439, -182, 0, -439, 0], [0, 820, 0, 0, 0, -180, 0, 0, 0, 0, 120, 0], [5005, 0, -5005, 0, -117, 0, -117, 0, 0, 173, 0, -173], [-820, 0, 0, 0, 762, 0, 0, 0, 0, 0, 0, 368], [-1481, 0, 0, 0, -541, 0, 0, 0, 0, 0, 0, -65], [0, -1081, 0, 0, 0, 170, 0, 0, 0, 0, 147, 0], [0, -1125, 0, 1125, 0, 0, 0, 147, 4, 0, 0, 0], [0, 0, -1733, 0, 0, 0, 708, 0, 0, 514, 0, 0], [0, 1331, 0, 0, 0, 8, 0, 0, 0, 0, 275, 0], [-3204, 0, 3204, 0, 122, 0, 122, 0, 0, -924, 0, 924], [0, 0, 0, 2323, 0, -58, 0, 481, 58, 0, -481, 0], [0, 0, 3748, 0, 0, 0, 222, 0, 0, 0, 0, 0], [0, 936, 0, -936, 0, 0, 0, 242, 22, 0, 0, 0], [2259, 0, -2259, 0, 1166, 0, 1166, 0, 0, 404, 0, -404], [0, 0, -2373, 0, 0, 0, -683, 0, 0, -1123, 0, 0], [0, 486, 0, -486, 0, 0, 0, -386, 112, 0, 0, 0], [0, 0, 0, -430, 0, 64, 0, -62, -64, 0, 62, 0], [-2837, 0, 0, 0, -934, 0, 0, 0, 0, 0, 0, 388], [0, -3441, 0, 0, 0, -74, 0, 0, 0, 0, -3, 0], [-4579, 0, 0, 0, 81, 0, 0, 0, 0, 0, 0, -1053], [0, 0, 0, -892, 0, -230, 0, 366, 230, 0, -366, 0], [0, -770, 0, 770, 0, 0, 0, 60, 330, 0, 0, 0], [-112, 0, 0, 0, 408, 0, 0, 0, 0, 0, 0, 876], [0, -1526, 0, 0, 0, 114, 0, 0, 0, 0, -224, 0], [0, 0, 8409, 0, 0, 0, -143, 0, 0, -967, 0, 0], [0, 774, 0, -774, 0, 0, 0, 322, 220, 0, 0, 0], [0, 1948, 0, -1948, 0, 0, 0, 472, -12, 0, 0, 0], [0, 0, 0, 1441, 0, -58, 0, -367, 58, 0, 367, 0], [-2855, 0, 2855, 0, -705, 0, -705, 0, 0, -225, 0, 225], [-13493, 0, 13493, 0, 114, 0, 114, 0, 0, 46, 0, -46], [-4887, 0, 4887, 0, -823, 0, -823, 0, 0, 63, 0, -63], [0, 4314, 0, -4314, 0, 0, 0, 70, -8, 0, 0, 0], [0, 0, 0, -1898, 0, 284, 0, 998, -284, 0, -998, 0], [0, 0, 0, 1872, 0, -90, 0, -162, 90, 0, 162, 0], [4859, 0, 0, 0, -1623, 0, 0, 0, 0, 0, 0, 103], [7372, 0, 0, 0, -490, 0, 0, 0, 0, 0, 0, 696], [0, 1037, 0, 0, 0, 158, 0, 0, 0, 0, 25, 0], [0, -1839, 0, 1839, 0, 0, 0, 597, -92, 0, 0, 0], [0, -393, 0, 393, 0, 0, 0, -609, -242, 0, 0, 0], [15597, 0, 0, 0, -350, 0, 0, 0, 0, 0, 0, -208], [8845, 0, 0, 0, 947, 0, 0, 0, 0, 0, 0, 443], [0, 0, 3260, 0, 0, 0, 950, 0, 0, -1260, 0, 0], [0, -1013, 0, 1013, 0, 0, 0, -659, 330, 0, 0, 0], [-10417, 0, 0, 0, -159, 0, 0, 0, 0, 0, 0, -181], [3717, 0, -3717, 0, 5, 0, 5, 0, 0, -763, 0, 763], [0, -160, 0, 0, 0, -274, 0, 0, 0, 0, 938, 0], [0, 0, -3671, 0, 0, 0, 585, 0, 0, 837, 0, 0], [0, 0, 0, 2598, 0, -132, 0, -498, 132, 0, 498, 0], [0, 0, 0, 1510, 0, 206, 0, -148, -206, 0, 148, 0], [0, 2357, 0, -2357, 0, 0, 0, -1245, -64, 0, 0, 0], [-12946, 0, 12946, 0, -344, 0, -344, 0, 0, 996, 0, -996], [-1435, 0, 0, 0, -726, 0, 0, 0, 0, 0, 0, -404], [0, 0, 0, -908, 0, 68, 0, -560, -68, 0, 560, 0], [0, 0, 0, 5547, 0, 330, 0, -75, -330, 0, 75, 0], [15932, 0, 0, 0, 1116, 0, 0, 0, 0, 0, 0, -48], [0, -645, 0, 0, 0, 496, 0, 0, 0, 0, 643, 0], [6181, 0, -6181, 0, 744, 0, 744, 0, 0, 1330, 0, -1330], [0, -383, 0, 0, 0, -372, 0, 0, 0, 0, 691, 0], [0, 0, 3955, 0, 0, 0, -1353, 0, 0, -1943, 0, 0], [5784, 0, 0, 0, 250, 0, 0, 0, 0, 0, 0, -768], [0, 0, -15829, 0, 0, 0, 563, 0, 0, -553, 0, 0], [0, 3693, 0, 0, 0, -350, 0, 0, 0, 0, 1325, 0], [-4551, 0, 4551, 0, -1063, 0, -1063, 0, 0, -463, 0, 463], [0, 0, 265, 0, 0, 0, -1560, 0, 0, 910, 0, 0], [0, 0, 0, -2765, 0, -298, 0, 659, 298, 0, -659, 0], [2657, 0, -2657, 0, -1796, 0, -1796, 0, 0, -916, 0, 916], [0, -3168, 0, 3168, 0, 0, 0, 1550, 346, 0, 0, 0], [575, 0, 0, 0, -111, 0, 0, 0, 0, 0, 0, -589], [0, 0, 0, 3846, 0, 402, 0, -136, -402, 0, 136, 0], [0, 0, 0, 115, 0, 152, 0, -711, -152, 0, 711, 0], [9928, 0, -9928, 0, 3492, 0, 3492, 0, 0, 20, 0, -20], [0, -3834, 0, 3834, 0, 0, 0, -1758, 472, 0, 0, 0], [-13584, 0, 13584, 0, 1532, 0, 1532, 0, 0, -2016, 0, 2016], [0, -1574, 0, 1574, 0, 0, 0, 1294, -252, 0, 0, 0], [15047, 0, 0, 0, -588, 0, 0, 0, 0, 0, 0, 1958], [0, 2013, 0, 0, 0, 34, 0, 0, 0, 0, -915, 0], [0, 0, -9519, 0, 0, 0, -2546, 0, 0, 162, 0, 0], [0, 2184, 0, 0, 0, -344, 0, 0, 0, 0, -1552, 0], [-2381, 0, 2381, 0, 669, 0, 669, 0, 0, -287, 0, 287], [0, 0, 0, -4760, 0, -130, 0, 1150, 130, 0, -1150, 0], [0, 0, -11867, 0, 0, 0, 366, 0, 0, 1970, 0, 0], [0, 0, 0, 581, 0, -164, 0, 2411, 164, 0, -2411, 0], [2105, 0, -2105, 0, -71, 0, -71, 0, 0, -1441, 0, 1441], [0, -5117, 0, 5117, 0, 0, 0, -1343, 276, 0, 0, 0], [0, 0, 21300, 0, 0, 0, -1784, 0, 0, 636, 0, 0], [0, 0, 0, 7600, 0, -178, 0, 1694, 178, 0, -1694, 0], [-1651, 0, 0, 0, 597, 0, 0, 0, 0, 0, 0, -2481], [0, -7085, 0, 0, 0, -492, 0, 0, 0, 0, -21, 0], [10327, 0, -10327, 0, -726, 0, -726, 0, 0, -812, 0, 812], [0, 844, 0, -844, 0, 0, 0, 2068, -672, 0, 0, 0], [0, -4946, 0, 0, 0, 222, 0, 0, 0, 0, 1040, 0], [0, -1921, 0, 0, 0, 20, 0, 0, 0, 0, -713, 0], [0, 0, -2835, 0, 0, 0, -491, 0, 0, 3579, 0, 0], [0, -936, 0, 0, 0, 886, 0, 0, 0, 0, -1638, 0], [0, 0, 0, -372, 0, 246, 0, -1782, -246, 0, 1782, 0], [0, -2483, 0, 0, 0, 540, 0, 0, 0, 0, -1965, 0], [0, 0, 13497, 0, 0, 0, 2545, 0, 0, 4643, 0, 0], [12673, 0, -12673, 0, 2286, 0, 2286, 0, 0, 1502, 0, -1502]]
